module FunctionsAndConstants where

import Data.Complex
import DataTypes
import Graphics.GD



beta = 1e-2
gama = 1e-3
delta = 1e-4
epsilon = 1e-5



--The maximum number of iterations
maxIteration :: Int
maxIteration = 100


---------------------------------------------------------------------------------------------------- COLOR FUNCTIONS
-- This function determines the color of the picture, for each i (number of iteration)
-- NOTE, if rgb value is over 256 it will give black color (use `mod` if you want to avoid this)
-- 1. variable is for red, 2. for green, 3. for blue color, 0 0 0 is black, and 255 255 255 is white
colorCalculate1 :: Int -> Color
colorCalculate1 i = rgb ((i^2+i*2) `mod` 256) ((i^3) `mod` 256) ((i*8) `mod` 256)


-- the 2 functions below are ment to be used mainly for the mix function, where you can take the number of iterations of each pixel for all the functions that you took in the mix function
colorCalculate2 :: Int -> Int -> Color
colorCalculate2 m n = rgb (m^2+m) (n*8+50) (n*8)


colorCalculate3 :: Int -> Int -> Int -> Color
colorCalculate3 a b c = rgb (a^2-c) (b*8+50-a) (c*3+150)






-- rotate is actually a matrix transformation, which takes a vector (from the origin to the point)and rotates it by angle phi in anticlockwise direction
rotate ::Complex Double -> Double -> Complex Double
rotate (x :+ y) phi = ((x* (cos phi) - y*(sin phi)) :+ (x*(sin phi) + y*(cos phi)))






------------------------------------------------------------------------------------------------------------- MANDOLBROT SET 
-- z changes, c should not !!!			(c is out point in the plane and z0 is set to (0,0))
mandelfractal :: RealFloat a => Complex a ->Complex a -> Int -> Int
mandelfractal z c iter 
		| iter >= maxIteration 	= 255
		| magnitude z < 2 	= mandelfractal z' c (iter+1)
		| otherwise 		= iter
		where 
		z' =  z^2 + c


------------------------------------------------------------------------------------------------------------------ JULIA SET 
-- z changes, c should not !!!			(c is choosen, z0 is our point in the plane)
juliafractal :: RealFloat a => Complex a -> Complex a -> Int -> Int
juliafractal z c iter 
		| iter >= maxIteration 	= 255
		| magnitude z < 2 	= juliafractal z' c (iter+1)
		| otherwise 		= iter
		where 
		z' =  z^2 + c



------------------------------------------------------------------------------------------------------------------FUNCTION C 
functionC1 :: Complex Double -> (Double, Double)	-- mangnitude, angle
functionC1 z =  ((magnitude z' / magnitude z ), (2* (atan (imagPart z' / realPart z'))))
		where z' = (z^2 - (1 :+ 0)) * (z - (2 :+ 1)) * 2/(z^2 + (2 :+ 2))
--magnitude of z' or magnitude of z' / magnitude z 
			 

colorCalculateC :: (Double, Double) -> Color
colorCalculateC	(x,y) 					--- change the first paragraph the *x part					-- RANGE OF COLORS	
			| x == 0 = rgb 122 122 122
			| x >= 1 && y <= -2/3 * pi 	= rgb 255 (round (((pi + y)*3.0*255.0/pi)*x)) white				-- RED TO YELLOW
			| x >= 1 && y <= -1/3 * pi	= rgb (255 - (round ((((2/3)*pi + y)*3.0*255.0/pi)*x))) 255 white		-- YELLOW TO GREEN		to white (change to 255)
			| x >= 1 && y <= 0		= rgb white 255 (round ((((1/3)*pi + y)*3.0*255.0/pi)*x))			-- GREEN TO CYAN
			| x >= 1 && y <= 1/3 * pi	= rgb white (255 - (round ((y*3.0*255.0/pi)*x))) 255				-- CYAN TO BLUE
			| x >= 1 && y <= 2/3 * pi	= rgb (round ((((y - (1/3)*pi)*3.0*255.0/pi)*x))) white 255			-- BLUE TO MAGENTA
			| x >= 1 && y <= pi		= rgb 255 white (255 - (round (((y - (2/3 * pi))*3.0*255.0/pi)*x)))		-- MAGENTA TO RED

			| x < 1 && y <= -2/3 * pi 	= rgb black (round (((pi + y)*3.0*255.0/pi)*x)) 0				-- RED TO YELLOW
			| x < 1 && y <= -1/3 * pi	= rgb (255 - (round ((((2/3)*pi + y)*3.0*255.0/pi)*x))) black 0			-- YELLOW TO GREEN
			| x < 1 && y <= 0		= rgb 0 black (round ((((1/3)*pi + y)*3.0*255.0/pi)*x))				-- GREEN TO CYAN		to black (change to 0)
			| x < 1 && y <= 1/3 * pi	= rgb 0 (255 - (round ((y*3.0*255.0/pi)*x))) black				-- CYAN TO BLUE
			| x < 1 && y <= 2/3 * pi	= rgb (round (((y - (1/3)*pi)*3.0*255.0/pi)*x)) 0 black				-- BLUE TO MAGENTA
			| x < 1 && y <= pi		= rgb black 0 (255 - (round (((y - (2/3 * pi))*3.0*255.0/pi)*x)))		-- MAGENTA TO RED
			where 
			black = (round (255.0*x))
			white = if x >= 255 then 255 else round x
			




---------------------------------------------------------------------------------------------------------------------- NEWTON1
newtonfractal1 :: Complex Double -> Int -> Int
newtonfractal1 z iter 
		| iter >= maxIteration 	= 255
		| abs (magnitude z - magnitude z') > delta = newtonfractal1 z' (iter+1)
		| otherwise 		= iter
		where 
		z' = z - m*(function1 z / function1' z)
		m = (1.5 :+ 0.75)


-- NOTE, complex functions (especially polynomials) do not follow the same rule as real functions
function1 x = x^3 - 1
function1' (x :+ y) = ((3* (x^2 - y^2)) :+ (6*x*y))




