module Main 	where				
import Data.Complex					
import Graphics.GD
import Data.Time (getCurrentTime)			
import System.Locale (defaultTimeLocale)
import Data.Time.Format (formatTime)
import System.Environment
import DataTypes
import FunctionsAndConstants






--- VARIABLES TO PLAY WITH


-- Size of the picture (preferable 320 x 320)
pictureSize :: PictureSize
pictureSize = (320, 320)


-- Part of the plane (used for zooming in)

planeTaken :: PlaneSize -- botton left, top right  
planeTaken = ((pReal - r, pImag - r), (pReal + r, pImag + r))


-- 2r is the length and height of the plane, while pReal is the parameter of the center point on the real axis and pImag is the parameter of the center point on the imaginary axis
r = 5
pReal = 0 	-- real axis
pImag = 0	-- imag axis







-- FUNCTIONS THAT CAN BE CHANGED


mandel :: Coordinate -> Color
mandel (x,y) = colorCalculate1 $ mandelfractal (0 :+ 0) (x :+ y) 0


julia :: Coordinate -> Coordinate -> Color
julia (a,b) (x,y) = colorCalculate1 $ juliafractal (x :+ y) (a :+ b) 0	


functionC :: Coordinate -> Color
functionC (x,y) = colorCalculateC (functionC1 (x :+ y)) 


newton :: Coordinate -> Color
newton (x,y) = colorCalculate1 $ newtonfractal1 (rotate (x :+ y) pi) 0



-- MIX (for fun)
-- you can choose more functions, where each will give you 1 Int value for each pixel, then you can use "colorCalculate (2 or 3)" to mix-up some nice colors (good luck!)
-- also to note, using the same functions twice with slight differences you can obtain a picture of mirror image or transformation of one set, etc.
mix :: Coordinate -> Color
mix (x,y) = colorCalculate1 ( (newtonfractal1 (rotate (x :+ y) pi) 0)  + (newtonfractal1 (x :+ y) 0))










-- FUNCTIONS NOT TO PLAY WITH


-- Gives the list of all the pixel, each assigned with a distinguished coordinate
pixels :: PictureSize -> [Pixel]
pixels (sizeX, sizeY) = [ (x,y) | x <- [0,1..sizeX - 1], y <- [0,1..sizeY - 1]]


-- Connects each pixel on the screen with a coordinate (from the plane) 
pixelToCoordinate :: Pixel -> PictureSize -> PlaneSize -> Coordinate
pixelToCoordinate (intX, intY) (length, height) ((a, b), (c, d)) = (x,y)
	where 	x = a + (fromIntegral intX) * (abs (a-c))/(fromIntegral length)
		y = b + (fromIntegral intY) * (abs (b-d))/(fromIntegral height)


-- This is the main drawing function. When given The size of the picture, it will create an empty picture and will bind it to "picture". Then mapM_ is performed to map each pixel with a set function (e.g. mandel) and then binding the result of set function for each pixel onto the "picture". Out picture will then be saved in the file "Pictures" as the time when it was created.


draw :: PictureSize -> PlaneSize -> String -> (Coordinate -> Color)-> IO ()
draw pictureSize planeSize pictureName set = 
	do 	picture <- newImage pictureSize
		let pixS = pixels pictureSize
		mapM_ (c picture) pixS
		savePngFile ("Pictures/" ++ pictureName) picture
	where c picture p = (setPixel p (set (pixelToCoordinate p pictureSize planeSize)) picture)
			


main :: IO () 
main = do
	 args <- getArgs
   	 case args !! 0 of 
		"mandel" -> do
			time <- getCurrentTime
			draw pictureSize planeTaken ((formatTime defaultTimeLocale "%c" time) ++ ".png") mandel 
		"julia" -> do
			time <- getCurrentTime
			draw pictureSize planeTaken ((formatTime defaultTimeLocale "%c" time) ++ ".png") (julia ((read (args !! 1) :: Double),(read (args !! 2) :: Double)) )
		"functionC" -> do
			time <- getCurrentTime
			draw pictureSize planeTaken ((formatTime defaultTimeLocale "%c" time) ++ ".png") functionC
		"newton" -> do
			time <- getCurrentTime
			draw pictureSize planeTaken ((formatTime defaultTimeLocale "%c" time) ++ ".png") newton 
		"mix" -> do
			time <- getCurrentTime
			draw pictureSize planeTaken ((formatTime defaultTimeLocale "%c" time) ++ ".png") mix 
		
